#import wedap
#import matplotlib.pyplot as plt
#import numpy as np

import matplotlib
matplotlib.use('agg')

from wedap.tests.test_h5_plot import plot_data_gen

h5 = "wedap/data/p53.h5"

# # 1 dataset plots
# for dt in ["evolution"]:
#     for pm in ["hist"]:
#         for x in ["pcoord", "dihedral_2"]:
#             plot_data_gen(h5=h5, data_type=dt, plot_mode=pm, Xname=x, #show=True)
#                           out=f"wedap/tests/data/plot_{dt}_{pm}_{x}.npy")
# for dt in ["average", "instant"]:
#     for pm in ["line"]:
#         for x in ["pcoord", "dihedral_2"]:
#             plot_data_gen(h5=h5, data_type=dt, plot_mode=pm, Xname=x, #show=True)
#                           out=f"wedap/tests/data/plot_{dt}_{pm}_{x}.npy")

# 2 dataset plots
# for dt in ["average", "instant"]:
#     for pm in ["hist", "contour"]:
#         for x, y in [["pcoord", "dihedral_2"], ["dihedral_2", "pcoord"]]:
#             plot_data_gen(h5=h5, data_type=dt, plot_mode=pm, Xname=x, Yname=y, #show=True) 
#                           out=f"wedap/tests/data/plot_{dt}_{pm}_{x}_{y}.npy")

# # 3 dataset plots
# for dt in ["average", "instant"]:
#     for pm in ["scatter3d", "hexbin3d"]:
#         for x, y, z in [["pcoord", "dihedral_2", "dihedral_3"], 
#                         ["dihedral_2", "pcoord", "dihedral_3"], 
#                         ["dihedral_2", "dihedral_3", "pcoord"]]:
#             plot_data_gen(h5=h5, data_type=dt, plot_mode=pm, Xname=x, Yname=y, Zname=z, 
#                           out=f"wedap/tests/data/plot_{dt}_{pm}_{x}_{y}_{z}.npy")
#             #plot_data_gen(h5=h5, data_type=dt, plot_mode=pm, Xname=x, Yname=y, Zname=z, show=True)

# tests for evolution arg options
# for dt in ["evolution"]:
#     for pm in ["hist"]:
#         # tests for first_iter, last_iter, step_iter
#         for fi, li, si in [[1, 15, 1], 
#                            [3, None, 1], 
#                            [5, 15, 3]]:
#             plot_data_gen(h5=h5, data_type=dt, plot_mode=pm, first_iter=fi, last_iter=li, step_iter=si, 
#                          out=f"wedap/tests/data/plot_{dt}_{pm}_fi{fi}_li{li}_si{si}.npy")
#             #plot_data_gen(h5=h5, data_type=dt, plot_mode=pm, first_iter=fi, last_iter=li, step_iter=si, show=True)

#         # test bins and hrx
#         bins = 50
#         hrx = [0, 8]
#         plot_data_gen(h5=h5, data_type=dt, plot_mode=pm, bins=bins, histrange_x=hrx,
#                      out=f"wedap/tests/data/plot_{dt}_{pm}_bins{bins}_hrx{hrx[0]}-{hrx[1]}.npy")
#         #plot_data_gen(h5=h5, data_type=dt, plot_mode=pm, bins=bins, histrange_x=hrx, show=True)

# tests for average arg options
# for dt in ["average"]:
#     for pm in ["hist"]:
#         # tests for first_iter, last_iter, step_iter
#         for fi, li, si in [[1, 15, 1], 
#                            [3, None, 1], 
#                            [5, 15, 3]]:
#             plot_data_gen(h5=h5, data_type=dt, plot_mode=pm, first_iter=fi, last_iter=li, step_iter=si, 
#                           Yname="pcoord", Yindex=1, #show=True)
#                           out=f"wedap/tests/data/plot_{dt}_{pm}_fi{fi}_li{li}_si{si}.npy")

#         # test bins and hrx/hry
#         bins = 50
#         hrx = [0, 8]
#         hry = [5, 35]
#         plot_data_gen(h5=h5, data_type=dt, plot_mode=pm, bins=bins, histrange_x=hrx, histrange_y=hry,
#                       Yname="pcoord", Yindex=1, #show=True)
#                       out=f"wedap/tests/data/plot_{dt}_{pm}_bins{bins}_hrx{hrx[0]}-{hrx[1]}_hry{hry[0]}-{hry[1]}.npy")


##########################################
################## TODO ##################
##########################################
# # TODO: tests for p_units (all versions - maybe better with pdist test) and with --weighted
# # TODO: tests for 3d and 4d proj plots
# 2 dataset plots with joint plots (TODO)
# for jp in [True, False]:
#     for dt in ["average", "instant"]:
#         for pm in ["hist", "contour"]:
#             for x, y in [["pcoord", "dihedral_2"], ["dihedral_2", "pcoord"]]:
#                 plot_data_gen(h5=h5, data_type=dt, plot_mode=pm, Xname=x, Yname=y,, jointplot=jp, out=f"wedap/tests/data/plot_{dt}_{pm}_{x}_{y}_jp{jp}.npy")


"""
Unit and regression tests for the MD_Pdist class.
"""

# Import package, test suite, and other packages as needed
import mdap

import numpy as np
import pytest

# look at file coverage for testing
# pytest -v --cov=wedap
# produces .coverage binary file to be used by other tools to visualize 
# do not need 100% coverage, 80-90% is very high

# can have report in 
# $ pytest -v --cov=wedap --cov-report=html
# index.html to better visualize the test coverage

# decorator to skip in pytest
#@pytest.mark.skip

# TODO: test for trace, search_aux, skip_basis, get_total_data_array, get_all_weights
# maybe test more args like first_iter, last_iter, step_iter, H5save_out, data_proc, bins, histrange, p_units

class Test_MD_Pdist():
    """
    Test each method of the MD_Pdist class.
    """
    data_path = "mdap/tests/data/"
    
    @pytest.mark.parametrize("Xname", ["input_data_0.dat", "input_data_2.npy", "input_data_2.pkl"])
    def test_timeseries(self, Xname):
        X, Y, _ = mdap.MD_Pdist(Xname=self.data_path + Xname, data_type="time").pdist()

        # X data is just the frames / timescale
        np.testing.assert_allclose(X, np.loadtxt(f"mdap/tests/data/timeseries_{Xname}_X.txt"))

        # Y data is the data value at each frame
        np.testing.assert_allclose(Y, np.loadtxt(f"mdap/tests/data/timeseries_{Xname}_Y.txt"))
        
    @pytest.mark.parametrize("Xname", ["input_data_3.dat"])
    @pytest.mark.parametrize("Xindex", [1, 2])
    @pytest.mark.parametrize("Xinterval", [1, 10])
    def test_pdist_1d(self, Xname, Xindex, Xinterval):
        X, Y, _ = mdap.MD_Pdist(data_type="pdist", Xname=self.data_path + Xname, 
                                Xindex=Xindex, Xinterval=Xinterval).pdist()
        np.testing.assert_allclose(X, np.loadtxt(f"mdap/tests/data/pdist1d_{Xname}_idx{Xindex}_int{Xinterval}_X.txt"))
        np.testing.assert_allclose(Y, np.loadtxt(f"mdap/tests/data/pdist1d_{Xname}_idx{Xindex}_int{Xinterval}_Y.txt"))

    @pytest.mark.parametrize("Xname", ["input_data_0.dat", "input_data_2.npy", "input_data_2.pkl"])
    @pytest.mark.parametrize("Yname", ["input_data_3.dat"])
    @pytest.mark.parametrize("Yindex", [1, 2])
    def test_pdist_2d(self, Xname, Yname, Yindex):
        X, Y, Z = mdap.MD_Pdist(data_type="pdist", Xname=self.data_path + Xname, Xinterval=10,
                                Yname=self.data_path + Yname, Yindex=Yindex).pdist()
        np.testing.assert_allclose(X, 
            np.loadtxt(f"mdap/tests/data/pdist2d_{Xname}_{Yname}_yidx{Yindex}_X.txt"))
        np.testing.assert_allclose(Y, 
            np.loadtxt(f"mdap/tests/data/pdist2d_{Xname}_{Yname}_yidx{Yindex}_Y.txt"))
        np.testing.assert_allclose(Z, 
            np.loadtxt(f"mdap/tests/data/pdist2d_{Xname}_{Yname}_yidx{Yindex}_Z.txt"))

    @pytest.mark.parametrize("Xname", ["input_data_0.dat"])
    @pytest.mark.parametrize("Yname", ["input_data_1.dat", "input_data_2.npy"])
    @pytest.mark.parametrize("Zname", ["input_data_2.dat", "input_data_2.pkl"])
    def test_pdist_3d(self, Xname, Yname, Zname):
        X, Y, Z = mdap.MD_Pdist(data_type="pdist", Xname=self.data_path + Xname, 
                                Yname=self.data_path + Yname, Zname=self.data_path + Zname).pdist()
        np.testing.assert_allclose(X, 
            np.loadtxt(f"mdap/tests/data/pdist3d_{Xname}_{Yname}_{Zname}_X.txt"))
        np.testing.assert_allclose(Y, 
            np.loadtxt(f"mdap/tests/data/pdist3d_{Xname}_{Yname}_{Zname}_Y.txt"))
        np.testing.assert_allclose(Z, 
            np.loadtxt(f"mdap/tests/data/pdist3d_{Xname}_{Yname}_{Zname}_Z.txt"))



import mdap
import numpy as np

data_path = "mdap/tests/data/"

# # timeseries
# for Xname in ["input_data_0.dat", "input_data_2.npy", "input_data_2.pkl"]:
#     X, Y, _ = mdap.MD_Pdist(Xname=data_path + Xname, data_type="time").pdist()
#     np.savetxt(f"mdap/tests/data/timeseries_{Xname}_X.txt", X)
#     np.savetxt(f"mdap/tests/data/timeseries_{Xname}_Y.txt", Y)

# # 1D pdists
# for Xname in ["input_data_3.dat"]:
#     for Xindex in [1, 2]:
#         for Xinterval in [1, 10]:
#             X, Y, _ = mdap.MD_Pdist(data_type="pdist", Xname=data_path + Xname, 
#                                     Xindex=Xindex, Xinterval=Xinterval).pdist()
#             np.savetxt(f"mdap/tests/data/pdist1d_{Xname}_idx{Xindex}_int{Xinterval}_X.txt", X)
#             np.savetxt(f"mdap/tests/data/pdist1d_{Xname}_idx{Xindex}_int{Xinterval}_Y.txt", Y)

# # 2D pdists
# for Xname in ["input_data_0.dat", "input_data_2.npy", "input_data_2.pkl"]:
#     for Yname in ["input_data_3.dat"]:
#         for Yindex in [1, 2]:
#             X, Y, Z = mdap.MD_Pdist(data_type="pdist", Xname=data_path + Xname, Xinterval=10,
#                                     Yname=data_path + Yname, Yindex=Yindex).pdist()
#             np.savetxt(f"mdap/tests/data/pdist2d_{Xname}_{Yname}_yidx{Yindex}_X.txt", X)
#             np.savetxt(f"mdap/tests/data/pdist2d_{Xname}_{Yname}_yidx{Yindex}_Y.txt", Y)
#             np.savetxt(f"mdap/tests/data/pdist2d_{Xname}_{Yname}_yidx{Yindex}_Z.txt", Z)
        
# 3D scatter
for Xname in ["input_data_0.dat"]:
    for Yname in ["input_data_1.dat", "input_data_2.npy"]:
        for Zname in ["input_data_2.dat", "input_data_2.pkl"]:
            X, Y, Z = mdap.MD_Pdist(data_type="pdist", 
                                    Xname=data_path + Xname, 
                                    Yname=data_path + Yname, 
                                    Zname=data_path + Zname).pdist()
            np.savetxt(f"mdap/tests/data/pdist3d_{Xname}_{Yname}_{Zname}_X.txt", X)
            np.savetxt(f"mdap/tests/data/pdist3d_{Xname}_{Yname}_{Zname}_Y.txt", Y)
            np.savetxt(f"mdap/tests/data/pdist3d_{Xname}_{Yname}_{Zname}_Z.txt", Z)


"""
Unit and regression tests for the H5_Plot class.
"""

# Import package, test suite, and other packages as needed
import wedap

import numpy as np
import pytest

import matplotlib
matplotlib.use('agg')

# look at file coverage for testing
# pytest -v --cov=wedap
# produces .coverage binary file to be used by other tools to visualize 
# do not need 100% coverage, 80-90% is very high

# can have report in 
# $ pytest -v --cov=wedap --cov-report=html
# index.html to better visualize the test coverage

# decorator to skip in pytest
#@pytest.mark.skip

def plot_data_gen(out=None, show=False, **kwargs):
    """
    Make plot and return or convert to npy binary data file.
    """
    plot = wedap.H5_Plot(**kwargs)
    plot.plot()
    fig = plot.fig

    if show:
        matplotlib.pyplot.show()
        return

    # draw the figure 
    # tight since canvas is large
    fig.tight_layout(pad=0)
    fig.canvas.draw()

    # Now we can save it to a numpy array.
    data = np.frombuffer(fig.canvas.tostring_rgb(), dtype=np.uint8)
    data = data.reshape(fig.canvas.get_width_height()[::-1] + (3,))
    
    # save as np binary file
    if out:
        np.save(out, data)
    
    matplotlib.pyplot.close()

    return data

class Test_H5_Plot():
    """
    Test each method of the H5_Plot class.
    """
    h5 = "wedap/data/p53.h5"
    
    @pytest.mark.parametrize("data_type", ["evolution"])
    @pytest.mark.parametrize("Xname", ["pcoord", "dihedral_2"])
    def test_1d_evolution_plots(self, data_type, Xname):
        # make plot data array
        plotdata = plot_data_gen(h5=self.h5, data_type=data_type, plot_mode="hist", Xname=Xname)

        # compare to previously generated plot data
        data = np.load(f"wedap/tests/data/plot_{data_type}_hist_{Xname}.npy")
        #np.testing.assert_allclose(plotdata, data)
        # check to see if the amount of mismatches is less than 500 (<1% of 1 million items)
        assert data.size - np.count_nonzero(plotdata==data) < 500

    @pytest.mark.parametrize("data_type", ["average", "instant"])
    @pytest.mark.parametrize("Xname", ["pcoord", "dihedral_2"])
    def test_1d_avg_inst_plots(self, data_type, Xname):
        # make plot data array
        plotdata = plot_data_gen(h5=self.h5, data_type=data_type, plot_mode="line", Xname=Xname)

        # compare to previously generated plot data
        data = np.load(f"wedap/tests/data/plot_{data_type}_line_{Xname}.npy")
        #np.testing.assert_allclose(plotdata, data)
        # check to see if the amount of mismatches is less than 500 (<1% of 1 million items)
        assert data.size - np.count_nonzero(plotdata==data) < 500
        

    #@pytest.mark.parametrize("jointplot", [True, False]) # TODO
    @pytest.mark.parametrize("data_type", ["average", "instant"])
    @pytest.mark.parametrize("plot_mode", ["hist", "contour"])
    @pytest.mark.parametrize("Xname, Yname", [["pcoord", "dihedral_2"], ["dihedral_2", "pcoord"]])
    def test_2_dataset_plots(self, data_type, plot_mode, Xname, Yname):#, jointplot):
        # make plot data array
        plotdata = plot_data_gen(h5=self.h5, data_type=data_type, plot_mode=plot_mode, 
                                 Xname=Xname, Yname=Yname)#, jointplot=jointplot)

        # compare to previously generated plot data
        data = np.load(f"wedap/tests/data/plot_{data_type}_{plot_mode}_{Xname}_{Yname}.npy")
        #data = np.load(f"wedap/tests/data/plot_{data_type}_{plot_mode}_{Xname}_{Yname}_jp{jointplot}.npy")
        #np.testing.assert_allclose(plotdata, data)
        # check to see if the amount of mismatches is less than 500 (<1% of 1 million items)
        assert data.size - np.count_nonzero(plotdata==data) < 500
    

    @pytest.mark.parametrize("plot_mode", ["scatter3d", "hexbin3d"])
    @pytest.mark.parametrize("data_type", ["average", "instant"])
    @pytest.mark.parametrize("Xname, Yname, Zname", [["pcoord", "dihedral_2", "dihedral_3"], 
                                                     ["dihedral_2", "pcoord", "dihedral_3"], 
                                                     ["dihedral_2", "dihedral_3", "pcoord"]])
    def test_3_dataset_plots(self, data_type, plot_mode, Xname, Yname, Zname):
        # make plot data array
        plotdata = plot_data_gen(h5=self.h5, data_type=data_type, plot_mode=plot_mode, 
                                 Xname=Xname, Yname=Yname, Zname=Zname)

        # compare to previously generated plot data
        data = np.load(f"wedap/tests/data/plot_{data_type}_{plot_mode}_{Xname}_{Yname}_{Zname}.npy")
        #np.testing.assert_allclose(plotdata, data)
        # check to see if the amount of mismatches is less than 500 (<1% of ~1 million items)
        assert data.size - np.count_nonzero(plotdata==data) < 500

    @pytest.mark.parametrize("first_iter, last_iter, step_iter", [[1, 15, 1], 
                                                                  [3, None, 1], 
                                                                  [5, 15, 3]])
    def test_evolution_fi_li_si(self, first_iter, last_iter, step_iter):
        # make plot data array
        plotdata = plot_data_gen(h5=self.h5, data_type="evolution", plot_mode="hist", 
                                 first_iter=first_iter, last_iter=last_iter, step_iter=step_iter)
        
        # compare to previously generated plot data
        data = np.load(f"wedap/tests/data/plot_evolution_hist_fi{first_iter}_li{last_iter}_si{step_iter}.npy")
        #np.testing.assert_allclose(plotdata, data)
        # check to see if the amount of mismatches is less than 500 (<1% of ~1 million items)
        assert data.size - np.count_nonzero(plotdata==data) < 500

    def test_evolution_bins_hrx(self, bins=50, hrx=[0, 8]):
        # make plot data array
        plotdata = plot_data_gen(h5=self.h5, data_type="evolution", plot_mode="hist", 
                                 bins=bins, histrange_x=hrx)
        
        # compare to previously generated plot data
        data = np.load(f"wedap/tests/data/plot_evolution_hist_bins{bins}_hrx{hrx[0]}-{hrx[1]}.npy")
        #np.testing.assert_allclose(plotdata, data)
        # check to see if the amount of mismatches is less than 500 (<1% of ~1 million items)
        assert data.size - np.count_nonzero(plotdata==data) < 500

    @pytest.mark.parametrize("first_iter, last_iter, step_iter", [[1, 15, 1], 
                                                                  [3, None, 1], 
                                                                  [5, 15, 3]])
    def test_average_fi_li_si(self, first_iter, last_iter, step_iter):
        # make plot data array
        plotdata = plot_data_gen(h5=self.h5, data_type="average", plot_mode="hist", 
                                 Yname="pcoord", Yindex=1,
                                 first_iter=first_iter, last_iter=last_iter, step_iter=step_iter)
        
        # compare to previously generated plot data
        data = np.load(f"wedap/tests/data/plot_average_hist_fi{first_iter}_li{last_iter}_si{step_iter}.npy")
        #np.testing.assert_allclose(plotdata, data)
        # check to see if the amount of mismatches is less than 500 (<1% of ~1 million items)
        assert data.size - np.count_nonzero(plotdata==data) < 500

    def test_average_bins_hrx_hry(self, bins=50, hrx=[0, 8], hry=[5, 35]):
        # make plot data array
        plotdata = plot_data_gen(h5=self.h5, data_type="average", plot_mode="hist", 
                                 Yname="pcoord", Yindex=1,
                                 bins=bins, histrange_x=hrx, histrange_y=hry)
        
        # compare to previously generated plot data
        data = np.load(f"wedap/tests/data/plot_average_hist_bins{bins}_hrx{hrx[0]}-{hrx[1]}_hry{hry[0]}-{hry[1]}.npy")
        #np.testing.assert_allclose(plotdata, data)
        # check to see if the amount of mismatches is less than 500 (<1% of ~1 million items)
        assert data.size - np.count_nonzero(plotdata==data) < 500


import wedap
import numpy as np

h5 = "wedap/data/p53.h5"

# for Xname in ["pcoord"]:
#     for Xindex in [0, 1]:
#         evolution = wedap.H5_Pdist(h5=h5, data_type="evolution", Xname=Xname, Xindex=Xindex)
#         X, Y, Z = evolution.pdist()

#         # X data is the variably filled array of instance pdist x values
#         np.savetxt(f"wedap/tests/data/evolution_{Xname}{Xindex}_X.txt", X)

#         # Z data is the pdist values of each iteration
#         np.savetxt(f"wedap/tests/data/evolution_{Xname}{Xindex}_Z.txt", Z)

# for Xname in ["pcoord"]:
#     for Yname in ["dihedral_3", "pcoord"]:
#         for Xindex in [0, 1]:
#             X, Y, Z = wedap.H5_Pdist(h5=h5, data_type="instant", Xindex=Xindex,
#                                      Xname=Xname, Yname=Yname).pdist()
#             np.savetxt(f"wedap/tests/data/instant_{Xname}{Xindex}_{Yname}_X.txt", X)
#             np.savetxt(f"wedap/tests/data/instant_{Xname}{Xindex}_{Yname}_Y.txt", Y)
#             np.savetxt(f"wedap/tests/data/instant_{Xname}{Xindex}_{Yname}_Z.txt", Z)

for Xname in ["dihedral_3", "pcoord"]:
    for Yname in ["pcoord"]:
        for Yindex in [0, 1]:
            X, Y, Z = wedap.H5_Pdist(h5=h5, data_type="average", Yindex=Yindex,
                                     Xname=Xname, Yname=Yname).pdist()
            np.savetxt(f"wedap/tests/data/average_{Xname}_{Yname}{Yindex}_X.txt", X)
            np.savetxt(f"wedap/tests/data/average_{Xname}_{Yname}{Yindex}_Y.txt", Y)
            np.savetxt(f"wedap/tests/data/average_{Xname}_{Yname}{Yindex}_Z.txt", Z)
